package de.mcs.tools.sps.mnemonic;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertTrue;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;

import de.mcs.tools.sps.exceptions.IllegalArgument;
import de.mcs.tools.sps.exceptions.SyntaxError;

class TestWAIT {

  private static final int BYTE_VALUE = 0x20;
  private final String MNEMONIC = "WAIT";

  @Test
  void testMnemonic() throws SyntaxError {
    WAIT mnemonic = new WAIT(MNEMONIC + " 0x00");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE, mnemonic.getByte());
    assertTrue(mnemonic.hasArgument());
    assertFalse(mnemonic.isUsingPage());

    mnemonic = new WAIT(MNEMONIC + " 0x01");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE + 1, mnemonic.getByte());

    mnemonic = new WAIT(MNEMONIC + " 0x0f");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE + 0x0f, mnemonic.getByte());

    mnemonic = new WAIT(MNEMONIC + " 1ms");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE, mnemonic.getByte());

    mnemonic = new WAIT(MNEMONIC + " 10ms");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE + 0x03, mnemonic.getByte());

    mnemonic = new WAIT(MNEMONIC + " 60s");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE + 0x0f, mnemonic.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      WAIT wait1 = new WAIT(MNEMONIC);
      wait1.checkArgument();
    });

    Assertions.assertThrows(IllegalArgument.class, () -> {
      WAIT wait1 = new WAIT(MNEMONIC + " 0x10");
      wait1.checkArgument();
    });

    Assertions.assertThrows(IllegalArgument.class, () -> {
      WAIT wait1 = new WAIT(MNEMONIC + " sieben");
      wait1.checkArgument();
    });

    Assertions.assertThrows(IllegalArgument.class, () -> {
      WAIT wait1 = new WAIT(MNEMONIC + " 12ms");
      wait1.checkArgument();
    });

    Assertions.assertThrows(IllegalArgument.class, () -> {
      WAIT wait1 = new WAIT(MNEMONIC + " 50s");
      wait1.checkArgument();
    });
  }

  @Test
  void testMnemonicFactory() throws SyntaxError {
    Mnemonic mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " 0x00", 0);
    assertEquals(WAIT.class, mnemonic.getClass());

    Assertions.assertThrows(SyntaxError.class, () -> {
      MnemonicFactory.getMnemonic(MNEMONIC, 0);
    });

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " 0x02", 0);
    assertEquals(BYTE_VALUE + 0x02, mnemonic.getByte());

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " 12", 0);
    assertEquals(BYTE_VALUE + 12, mnemonic.getByte());

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " 0b00001010", 0);
    assertEquals(BYTE_VALUE + 10, mnemonic.getByte());

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " 10ms", 0);
    assertEquals(BYTE_VALUE + 3, mnemonic.getByte());
  }
}
